import types
from numba import npyufunc
from numba.types import none
from .stream import StreamUFuncBuilder

def _patch_add(impl):
    '''
    Maintains backwards compatibility with the argtypes and restypes kwargs
    that were provided by the add() methods of vectorized functions.
    '''
    original_add = impl.add
    def add(self, sig=None, argtypes=None, restype=None):
        restype = restype or none
        sig = sig or restype(*argtypes)
        original_add(sig)

    impl.add = types.MethodType(add, impl)
    return impl


class Vectorize(npyufunc.Vectorize):
    def __new__(cls, func, **kws):
        impl = super(Vectorize, cls).__new__(cls, func, **kws)
        return _patch_add(impl)

class GUVectorize(npyufunc.GUVectorize):
    def __new__(cls, func, signature, **kws):
        impl = super(GUVectorize, cls).__new__(cls, func, signature, **kws)
        return _patch_add(impl)


vectorize = npyufunc.vectorize
guvectorize = npyufunc.vectorize

Vectorize.target_registry['stream'] = StreamUFuncBuilder
